"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const constants_1 = require("../constants");
const Object_1 = require("../types/Object");
class ObjectModule {
    //
    // Static methods
    //
    static create(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/objects`,
                data: {
                    objectType: Object_1.isWarrantObject(params.object) ? params.object.getObjectType() : params.object.objectType,
                    objectId: Object_1.isWarrantObject(params.object) ? params.object.getObjectId() : params.object.objectId,
                    meta: params.meta,
                },
                options,
            });
        });
    }
    static batchCreate(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.post({
                url: `/${constants_1.API_VERSION}/objects`,
                data: params.map((objectParams) => ({
                    objectType: Object_1.isWarrantObject(objectParams.object) ? objectParams.object.getObjectType() : objectParams.object.objectType,
                    objectId: Object_1.isWarrantObject(objectParams.object) ? objectParams.object.getObjectId() : objectParams.object.objectId,
                    meta: objectParams.meta,
                })),
                options,
            });
        });
    }
    static get(object, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const objectType = Object_1.isWarrantObject(object) ? object.getObjectType() : object.objectType;
            const objectId = Object_1.isWarrantObject(object) ? object.getObjectId() : object.objectId;
            return yield WarrantClient_1.default.httpClient.get({
                url: `/${constants_1.API_VERSION}/objects/${objectType}/${objectId}`,
                options,
            });
        });
    }
    static list(listParams, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield WarrantClient_1.default.httpClient.get({
                url: `/${constants_1.API_VERSION}/objects`,
                params: listParams,
                options,
            });
        });
    }
    static update(object, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const objectType = Object_1.isWarrantObject(object) ? object.getObjectType() : object.objectType;
            const objectId = Object_1.isWarrantObject(object) ? object.getObjectId() : object.objectId;
            return yield WarrantClient_1.default.httpClient.put({
                url: `/${constants_1.API_VERSION}/objects/${objectType}/${objectId}`,
                data: {
                    meta: params.meta,
                },
                options,
            });
        });
    }
    static delete(object, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const objectType = Object_1.isWarrantObject(object) ? object.getObjectType() : object.objectType;
            const objectId = Object_1.isWarrantObject(object) ? object.getObjectId() : object.objectId;
            const response = yield WarrantClient_1.default.httpClient.delete({
                url: `/${constants_1.API_VERSION}/objects/${objectType}/${objectId}`,
                options,
            });
            return response.warrantToken;
        });
    }
    static batchDelete(params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield WarrantClient_1.default.httpClient.delete({
                url: `/${constants_1.API_VERSION}/objects`,
                data: params.map((objectParams) => ({
                    objectType: Object_1.isWarrantObject(objectParams.object) ? objectParams.object.getObjectType() : objectParams.object.objectType,
                    objectId: Object_1.isWarrantObject(objectParams.object) ? objectParams.object.getObjectId() : objectParams.object.objectId,
                })),
                options,
            });
            return response.warrantToken;
        });
    }
}
exports.default = ObjectModule;
