"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ObjectModule_1 = __importDefault(require("./ObjectModule"));
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const ObjectType_1 = require("../types/ObjectType");
class Permission {
    constructor(permissionId, meta) {
        this.permissionId = permissionId;
        this.meta = meta;
    }
    //
    // Static methods
    //
    static create(permission = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.create({
                    object: {
                        objectType: ObjectType_1.ObjectType.Permission,
                        objectId: permission.permissionId,
                    },
                    meta: permission.meta,
                }, options);
                return new Permission(response.objectId, response.meta);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.get({
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                }, options);
                return new Permission(response.objectId, response.meta);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(permissionId, params, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.update({
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                }, { meta: params.meta }, options);
                return new Permission(response.objectId, response.meta);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return ObjectModule_1.default.delete({
                objectType: ObjectType_1.ObjectType.Permission,
                objectId: permissionId,
            }, options);
        });
    }
    static listPermissions(listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield ObjectModule_1.default.list(Object.assign({ objectType: ObjectType_1.ObjectType.Permission }, listParams), options);
                const permissions = response.results.map((object) => new Permission(object.objectId, object.meta));
                return Object.assign(Object.assign({}, response), { results: permissions });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listPermissionsForUser(userId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select permission where user:${userId} is *`, listParams, options);
                const permissions = queryResponse.results.map((queryResult) => new Permission(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: permissions });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPermissionToUser(userId, permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removePermissionFromUser(userId, permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static listPermissionsForRole(roleId, listParams = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const queryResponse = yield WarrantModule_1.default.query(`select permission where role:${roleId} is *`, listParams, options);
                const permissions = queryResponse.results.map((queryResult) => new Permission(queryResult.objectId, queryResult.meta));
                return Object.assign(Object.assign({}, queryResponse), { results: permissions });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignPermissionToRole(roleId, permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Role,
                    objectId: roleId,
                }
            }, options);
        });
    }
    static removePermissionFromRole(roleId, permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Permission,
                    objectId: permissionId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Role,
                    objectId: roleId,
                }
            }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Permission;
    }
    getObjectId() {
        return this.permissionId;
    }
}
exports.default = Permission;
